<iframe src="../.ibox.html?raw=true" style="border:none; position:fixed; width:40px; right:0; z-index=999;"></iframe>

# Semantics

The evaluation semantics of ogma is built upon the idea of _pipelines_ of data. The _output_ of a
block becomes the _input_ of the subsequent block. Multiple blocks taken together make up an
_expression_. When ogma goes to process an expression there are three distinct phases:
1. Parsing - the expression is parsed from a string of characters into a syntax tree,
2. Intermediate Checking - checks the types and semantics of commands, and produces a streamlined
   evaluation model
3. Evaluation - executes the evaluation model.

Errors will usually occur in the 2nd phase when types and semantics of commands are
checked. This is where a solid mental model of the evaluation is handy, understanding the
differences between the _syntax_ of ogma and the _semantics_. For example, say an
expression is defined as `\ 3 | if {= 3} #t 'foo'`. This is valid syntax, but the **semantics** of the
`if` command require that both branches return the _same type_. As such a semantics error is
returned:
```plaintext
>> \ 3 | if {= 3} #t 'foo'
Evaluation Error: branch arms do not have matching output types
--> shell:15
 | \ 3 | if {= 3} #t 'foo'
 |                ^^ this branch has a different output type
--> help: branching impls require consistent output types
```

With a mental model of _pipelines_, these semantics usually reduce to thinking of the _types_ going
into and out of a block. Taking the example before, each element can be annotated with types:
```plaintext
\ 3 | if {= 3} #t 'foo'
  |   |   |  | |  ^^^^^ literal := Str
  |   |   |  | ^^ literal := Bool
  |   |   |  ^ output := Bool
  |   |   ^ input := Num
  |   ^ input := Num
  ^ output := Num
```

If we think of the `if` command as having the following type semantics: `if <Bool> <a> <a>`
where the first argument is something **returning a `Bool`ean**, and the second and third
argument returning the **same type `a`** (where `a` is any type), then it is straightforward to
deduce that `if {= 3} #t 'foo'`, that is `if <Bool> <Bool> <Str>`, does not match `if`'s
constraints.

> This requirement for the branches of an if expression to be of the same type flows into further
> type checking. We know that no matter which path is taken, the type of `a` will be produced,
> which can be fed into the next block!

The semantics of nested sub-expressions is worth understanding as well. An example is again using
an `if` command: `if {= 'Hello'} {+ ', world!'} 'Hello, world!'`. In this example, if the input is
`Hello`, the **input** is concatenated with `, world!` (using the sub-expression `{+ ', world!'}`).
Note how the input is **_the same input_** which was tested for equality. This is true for all
arguments which are not acting as _closures_. The difference between a _closure_ expression and a
sub-expression is that **sub-expressions** get evaluated _before_ the value is used with the command,
whilst **closures** are evaluated at later stages. Closure expressions are used for table
transformations where the same closure is evaluated repeatedly with different `TableRow` inputs.

> A good technique to check that the input into the block is what is expected, test a _portion_ of
> the expression in the REPL and see what the output is.

