<iframe src="../.ibox.html?raw=true" style="border:none; position:fixed; width:40px; right:0; z-index=999;"></iframe>

# Typing

ogma is _strongly_ typed. This means that each node in the syntax tree has a known type at compile
time, and the correct type must be used as inputs and arguments.
ogma will try to guess the types being used in an expression using _type inference_, which, as the
name implies, will infer the types from the usage.
It is easy to show the type system throwing an error:
```plaintext
>> \ 2 | + 'a'
Semantics Error: expecting argument with output type `Number`, found `String`
--> shell:9
 | \ 2 | + 'a'
 |          ^ this argument returns type `String`
--> help: commands may require specific argument types, use `--help` to view requirements
```

You cannot add a string to a number!

The type system also works to _choose_ the correct operation.
For example, the `len` command will work on a table or a string:

```plaintext
>> \"Hello, world!" | len
13

>> Table | len
0

>> \3 | len
Semantics Error: `len` does not support `Number` input data
--> shell:5
 | \3 | len
 |      ^^^
--> help: use `len --help` to view requirements. consider implementing `def len`
```

## Type Inference
---

To guess the types from use, ogma must trial differing types and use deduction to land on a type
set which would fit and expression. This process is fallible, especially when the use would lead to
ambiguity:

```plaintext
>> Table | nth 0 get foo
Semantics Error: ambiguous inference. more than one output type can compile op
--> shell:14
 | Table | nth 0 get foo
 |               ^^^ this op can be compiled with `Nil` and `TableRow` as output types
--> shell:14
 | Table | nth 0 get foo
 |               ^^^ try annotating output type: `get:<type> ...`
```

In this example, the `get` call has to infer an output type, but this could be any type!
The inference algorithm is [backed by
graphs](https://www.kurtlawrence.info/blog/type-inference-in-ogma-using-graphs), and is quite
robust, but not omniscient!

> ℹ️  Need more type information? Use the `typify` command to annotate an expression with types:
> ```plaintext
> >> typify { Table | nth 0 get:Num foo }
> {:Nil Table |:Table nth 0 {:TableRow get foo }:Num }:Num
> ```

## Type Annotation
---

When the inference algorithm runs into trouble, there exists syntax to _annotate_ type information.
The annotation act as _constraints_ on the expression. In general, the annotation take a leading
colon and then the type name `:<type>`.
An annotation _before_ a command constrains the **input**, while an annotation _after_ a command
constrains the **output**. Expression outputs can also be annotated.

```plaintext
>> {:Num Table |:Table nth get:Num foo }:Num
    ^^^^        ^^^^^^        ^^^^      ^^^^
    |           |             |         | constrain expression output
    |		|	      | constrain command output
    |		| constrain command input (`nth command`)
    | constrain command input
```

Annotations can accidentally be applied which results in type conflicts:

```plaintext
>> Table |:Num + 3
Typing Error: Type resolution failed. Conflicting obligation type
--> shell:0
 | Table |:Num + 3
 | ^^^^^ this node returns a `Table`
--> shell:12
 | Table |:Num + 3
 |             ^ but this node is obliged to return `Number`
```
