<iframe src="../.ibox.html?raw=true" style="border:none; position:fixed; width:40px; right:0; z-index=999;"></iframe>

# Weight-averaging

A common pattern is to weight average values across a table. In this example the `diamonds` dataset
will be used to weight average.
A weight average can be achieved using a `fold` and is 
encapsulated in a fold implementation as shown below:
1. Extract the column value and weight value (using `let`),
2. Multiply the value and weight together,
3. Construct new tuple, adding multiplied value and weight to previous tuple,
4. Divide out the weight to get the average.
```plaintext
def wavg Table (col:Str weight:Str) { fold {Tuple 0 0} { 
    let $acc | \$row | let {get $col} $x {get $weight} $w 
    | * $x $w | Tuple {+ $acc.t0} {\$w | + $acc.t1}
} | / {get t0} {get t1} }

>> open diamonds.csv | wavg price carat
6116.840634262495
```

Because the weight average is encapsulated, it can be easily be used to reduce grouped tables as
well. Below is the weight average price grouped into colours:
```plaintext
>> open diamonds.csv | grp color | map value --Table { wavg price carat }
```
![](../assets/adv-pat.wavg.png?raw=true)
> Another way to define a weight-average implementation is to use `map` to store intermediate
> values. This might be more performant for very large tables.
>
> ```plaintext
> def wavg2 Table (col:Str weight:Str) {
>     map $col * { \$row | get $weight }
>     | let { fold 0 + { \$row | get $weight } } $w
>     | fold 0 + { \$row | get $col }
>     | / $w
> }
> ```
